@@ -4,13 +4,14 @@ use Carp;
 use Dancer2::Core::Types qw/Bool Int Str/;
 use DateTime;
 use DBIx::Class::ResultClass::HashRefInflator;
+use Scalar::Util qw(blessed);
 use String::CamelCase qw(camelize);
 
 use Moo;
 with "Dancer2::Plugin::Auth::Extensible::Role::Provider";
 use namespace::clean;
 
-our $VERSION = '0.603';
+our $VERSION = '0.620';
 
 =head1 NAME 
 
@@ -572,6 +573,8 @@ sub _user_rset {
 
 sub authenticate_user {
     my ($self, $username, $password, %options) = @_;
+    croak "username and password must be defined"
+      unless defined $username && defined $password;
 
     my ( $user ) = $self->_user_rset( 'username', $username )->all;
     return unless $user;
@@ -604,6 +607,9 @@ sub authenticate_user {
 
 sub set_user_password {
     my ( $self, $username, $password ) = @_;
+    croak "username and password must be defined"
+      unless defined $username && defined $password;
+
     my $encrypted       = $self->encrypt_password($password);
     my $password_column = $self->users_password_column;
     my %update          = ( $password_column => $encrypted );
@@ -617,7 +623,8 @@ sub set_user_password {
 # fetched and all columns returned as a hashref.
 sub get_user_details {
     my ($self, $username) = @_;
-    return unless defined $username;
+    croak "username must be defined"
+      unless defined $username;
 
     # Look up the user
     my $users_rs = $self->_user_rset(username => $username);
@@ -632,12 +639,22 @@ sub get_user_details {
         $self->plugin->app->log( 'debug', "No such user $username" );
         return;
     }
+
+    if ( !$self->user_as_object ) {
+        if ( my $roles_key = $self->roles_key ) {
+            my @roles = @{ $self->get_user_roles($username) };
+            my %roles = map { $_ => 1 } @roles;
+            $user->{$roles_key} = \%roles;
+        }
+    }
     return $user;
 }
 
 # Find a user based on a password reset code
 sub get_user_by_code {
     my ($self, $code) = @_;
+    croak "code needs to be specified"
+      unless $code && $code ne '';
 
     my ($user) = $self->_user_rset( pw_reset_code => $code )->all;
     return unless $user;
@@ -649,8 +666,10 @@ sub get_user_by_code {
 sub create_user {
     my ($self, %user) = @_;
     my $username_column = $self->users_username_column;
-    my $username        = delete $user{username} # Prevent attempt to update wrong key
-        or croak "Username needs to be specified for create_user";
+    my $username        = delete $user{username}; # Prevent attempt to update wrong key
+    croak "Username not supplied in args"
+      unless defined $username && $username ne '';
+
     $self->schema->resultset($self->users_resultset)->create({
         $username_column => $username
     });
@@ -669,19 +688,22 @@ sub set_user_details {
     $user or return;
 
     # Are we expecting a user_roles key?
-    if (my $roles_key = $self->roles_key) {
-        if (my $new_roles = delete $update{$roles_key}) {
+    if ( my $roles_key = $self->roles_key ) {
+        if ( my $new_roles = delete $update{$roles_key} ) {
 
             my $roles_role_column     = $self->roles_role_column;
             my $users_username_column = $self->users_username_column;
 
-            my @all_roles      = $self->schema->resultset($self->roles_resultset)->all;
-            my %existing_roles = map { $_ => 1 } @{$self->get_user_roles($username)};
+            my @all_roles =
+              $self->schema->resultset( $self->roles_resultset )->all;
+            my %existing_roles =
+              map { $_ => 1 } @{ $self->get_user_roles($username) };
 
             foreach my $role (@all_roles) {
                 my $role_name = $role->$roles_role_column;
 
-                if ( $new_roles->{$role_name} && !$existing_roles{$role_name} )
+                if ( $new_roles->{$role_name}
+                    && !$existing_roles{$role_name} )
                 {
                     # Needs to be added
                     $self->schema->resultset( $self->user_roles_resultset )
@@ -736,6 +758,8 @@ sub set_user_details {
 
 sub get_user_roles {
     my ($self, $username) = @_;
+    croak "username must be defined"
+      unless defined $username;
 
     my $role_relationship            = $self->role_relationship;
     my $user_user_roles_relationship = $self->user_user_roles_relationship;
@@ -764,6 +788,11 @@ sub get_user_roles {
 
 sub password_expired {
     my ($self, $user) = @_;
+    croak "user must be specified"
+      unless defined $user
+      && ( ref($user) eq 'HASH'
+        || ( blessed($user) && $user->isa('DBIx::Class::Row') ) );
+
     my $expiry   = $self->password_expiry_days or return 0; # No expiry set
 
     if (my $pwchanged = $self->users_pwchanged_column) {
